<?php

namespace Drupal\ai_media_data_fetch;

use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\GenericType\ImageFile;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Render\Element;
use Drupal\file\Entity\File;

class AiResponseHelper {

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public static function ajax(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $trigger = $form_state->getTriggeringElement();

    $image_field_values = $form_state->getValue($trigger["#ai_media_data_fetch"]["field_name"]);
    $fids = array_map(fn($image) => $image['fids'][0], $image_field_values);
    $files = File::loadMultiple($fids);

    $images = self::prepareFiles($files);

    $settings = $trigger["#ai_media_data_fetch"]["settings"];
    $system_role = $settings['system_role'] ?? NULL;
    $prompt = $settings['prompt'];

    $ai_provider = \Drupal::service('ai.provider')->createInstance('openai');
    $ai_provider->setConfiguration($settings['configuration']);
    $ai_provider->setChatSystemRole($system_role);

    $input = new ChatInput([new ChatMessage("user", $prompt, $images)]);
    $ai_response = $ai_provider->chat($input, $settings['model'], ["ai_media_data_fetch"])->getNormalized();

    $foobar = $ai_response->getText();
    $nice2have = Json::decode($foobar ?? '{ "field_media_image.alt": "A whimsical scene featuring small characters in a lush, animated environment with homes that have a sci-fi design.", "field_media_image.title": "Whimsical Animated Scene", "name": "Animated Character Gathering", "field_tags": ["animation", "fantasy", "characters", "sci-fi", "whimsical"] }');

    foreach ($nice2have as $key_path => $value) {
      $path_array = explode('.', $key_path);
      self::fieldRecursionHelper($form, $path_array, $value);
    }

    // @todo We should not replace the complete form. Better just replace the fields that have been changed.
    $response->addCommand(new ReplaceCommand("#media-image-edit-form", $form));
    $response->addCommand(new MessageCommand('Message', NULL, ['type' => 'status']));

    return $response;
  }

  /**
   * @param $files
   *
   * @return array
   */
  private static function prepareFiles($files): array {
    $images = [];

    foreach ($files as $file) {
      $image = new ImageFile();
      if ($image_style_name = true) {
        /** @var \Drupal\Image\Entity\ImageStyle */
        $entityTypeManager = \Drupal::entityTypeManager();
        $image_style = $entityTypeManager->getStorage('image_style')->load('large');
        $image_uri = $file->getFileUri();
        // Get the image style url and generate it.
        $scaled_image_uri = $image_style->buildUri($image_uri);
        $image_style->createDerivative($image_uri, $scaled_image_uri);
        $image->setBinary(file_get_contents($scaled_image_uri));
        $image->setMimeType('image/png');
      } else {
        $image->setFileFromFile($file);
      }

      $images[] = $image;
    }

    return $images;
  }

  /**
   * @param $element
   * @param $path_array
   * @param $value
   *
   * @return void
   */
  private static function fieldRecursionHelper(&$element, $path_array, $value) {
    if(empty($path_array) && isset($element['#value'])) {
      $element['#value'] = $value;
      return $element;
    }

    if(isset($element[$path_array[0]])) {
      self::fieldRecursionHelper($element[$path_array[0]], array_slice($path_array,1), $value);
    } else {
      foreach (Element::children($element) as $delta) {
        self::fieldRecursionHelper($element[$delta], $path_array, $value);
      }
    }
  }

}
